@@ -53,7 +53,9 @@ globalG.test <- function(x, listw, zero.policy=FALSE,
 		lower.tail=FALSE)
         else if (alternative == "greater")
             PrG <- pnorm(statistic, lower.tail=FALSE)
-        else PrI <- pnorm(statistic)
+        else PrG <- pnorm(statistic)
+	if (PrG < 0 || PrG > 1) 
+		warning("Out-of-range p-value: reconsider test arguments")
 	vec <- c(G, E.G, var.G)
 	names(vec) <- c("Global G statistic", "Expectation", "Variance")
 	data.name <- paste(deparse(substitute(x)), "\nweights:",
