@@ -1,8 +1,16 @@
-# Copyright 2001-4 by Roger S. Bivand. 
+# Copyright 2001-7 by Roger S. Bivand. 
+# Upgrade to sp classes February 2007
 #
 
-knearneigh <- function(x, k=1, lonlat=FALSE)
+knearneigh <- function(x, k=1, longlat=NULL)
 {
+    if (inherits(x, "SpatialPoints")) {
+        if ((is.null(longlat) || !is.logical(longlat)) 
+ 	   && !is.na(is.projected(x)) && !is.projected(x)) {
+           longlat <- TRUE
+        } else longlat <- FALSE
+        x <- coordinates(x)
+    } else if (is.null(longlat) || !is.logical(longlat)) longlat <- FALSE
     if (!is.numeric(x)) stop("Data non-numeric")
     if (!is.matrix(x)) stop("Data not in matrix form")
     if (any(is.na(x))) stop("Data include NAs")
@@ -17,7 +25,7 @@ knearneigh <- function(x, k=1, lonlat=FALSE)
     z <- .C("knearneigh", k=as.integer(k), np=as.integer(np),
         dimension=as.integer(dimension),
         xx=as.double(xx), nn=as.integer(nn), dnn=as.double(dnn),
-	as.integer(lonlat), PACKAGE="spdep")
+	as.integer(longlat), PACKAGE="spdep")
     res <- list(nn=matrix(z$nn, np, k, byrow=TRUE), np=np, k=k,
     	dimension=dimension, x=x)
     class(res) <- "knn"
